
function BasicScreenController(){
	var self = this; self.name = "BasicService"; // MANDATORY
	
	// Vars
	this.data = {
		internal_id: null, // Used in error control
        id : null,
        remote_id : null,
        hex_remote_id : null,
        model : null
    }
    this.status = {
        power : 0,
        channel : " ",
        lastCMD : " ",
        tstamp : 0
    }
    this.config = {
        schedule : null,
        delay : 0
    }

    this.intervals = {
            power : 20000,
            channel : 60000,
            schedule : 30000	
    }
    this.errCounter = {
        power : 0,
        channel : 0,
        schedule : 0
    };
    this.protocol = null;
    this.stopping = false;

    /* empty config */
    this.portValues = {
    		name : '',
    	    config : {
		            baudRate : 0,
		            dataBits : 0,
		            stopBits : 0
		    },
		    parser : {
	    		type : '' ,
	            delimiter : 0 
		    }	
    }
    
    
    /******************************** SPECIAL CHARACTERS ********************************/
    this.STX = Buffer.from ( '02' , 'hex');
    this.ETX = Buffer.from ( '03' , 'hex');
    
    /******************************** STATUS ********************************/
    this.getStatus = function () {
    	var status = false;
        status = { 
        	"int_id"	:	self.data.internal_id,
    		"id_screen"	:	self.data.id,
			"port"		:	self.portValues.name,
			"schedule"	:	{ 
								"delay"	:	self.config.delay,
								"halt"	:	false
							},
			"brand"		:	self.data.model,
			"remote_id"	:	self.data.remote_id,
			"status"	:	self.status.power,
			"channel"	:	self.status.channel,
			"lastCMD"	:	self.status.lastCMD,
			"tstamp"	:	self.status.tstamp,
			"errors"	:	0
        }
    	return status
    }
	    
    /*********************************** SCHEDULLE FUNCTIONS ***********************************/
    this.errorHandling = function ( err ) {
    	//console.log(err);
    	if      ( err.message.includes ( "Forced Time Out") ) { self.status.lastCMD = "Forced Time Out"; }
        else if ( err.message.includes ( "Cannot lock port") || err.message.includes ("Access den") ) { self.status.lastCMD = "Temporarily unavailable"; }
        else if ( err.message.includes ( "No such file or directory") || err.message.includes ("File not found") ) { self.status.lastCMD = "Port does not exist"; }
        else { self.status.lastCMD = err.message; }
    }
    
    /*********************************** SCHEDULLE FUNCTIONS ***********************************/

    this.getBinSchedule = function ( schedule) {
        var bufferSchedule = Buffer.from(schedule, "hex");
        var binSchedule = []
        for ( var i = 0; i < bufferSchedule.length; i++ ){
            binSchedule = binSchedule.concat (self.byte2Boolean (bufferSchedule[i]))
        }
        return binSchedule;
    },
    
    this.byte2Boolean = function ( byte ) {
        var ret = [];
        for (var i = 0; i < 8; i++) {
            ret[8 - 1 - i] = (1 << i & byte) != 0;
        }
        return ret;
    },
    
    this.getScheduledPower = function ( schedule , delay){
    	var date = self.getDate();
    	if (delay) date = new Date (date.getTime()-delay*1000*60) //change date in funtion of delay
        position = self.getPosition( date );
        return schedule[position]
    },
    
    this.getPosition = function( date ){
        var position = date.getDay();
        if(position == 0) position = 7; //correct day shift ( Sunday = 0 )
        position = ( position - 1 ) * 24; //get position of day
        position = position + date.getHours(); // add hour position
        return position
    };
    
    /*********************************** CHECKSUM FUNCTIONS ***********************************/

    this.getXorChecksum = function ( data ) {
        var checksum = Buffer.from ( '00' , 'hex');
        for (i = 0; i < data.length; i++)
        {
            checksum[0] ^= data[i];
        }
        return checksum
    }
    
    
    /*********************************** DATE FUNCTION ***********************************/
    
    this.getDate = function(utc,timestamp){
        if(typeof(timestamp) === "undefined"){ timestamp = -1; }
        if(typeof(utc) === "undefined"){ utc = false; }
        var systemDate = new Date();
        if(self.server_time_activated){
            var timezone_offset = 0;
            if(!utc){
                timezone_offset = self.server_time_timezone_offset*-1;
            }
            if(timestamp >= 0){
                timestamp = (timestamp+timezone_offset);
            }else{
                var system_timestamp = systemDate.getTime();
                if(self.platform == "SOC"){
                     var time_api = document.getElementById('timePlugin');
                     time_api.Open('Time', '1.000', 'Time');
                     system_timestamp = parseInt(time_api.Execute('GetEpochTime'));
                     time_api.Close();
                     time_api = null;
                     system_timestamp = system_timestamp * 1000;
                }
                timestamp = (system_timestamp+(self.server_time_local_offset*-1)+(timezone_offset));
            }
            systemDate = new Date(timestamp);
        }else{
             if(self.platform == "SOC"){
                 var time_api = document.getElementById('timePlugin');
                 time_api.Open('Time', '1.000', 'Time');
                 timestamp = parseInt(time_api.Execute('GetEpochTime'));
                 time_api.Close();
                 timestamp = timestamp * 1000;
                 var d = new Date(timestamp);
                 var offset = d.getTimezoneOffset();
                 offset = offset * 60;
                 offset = offset * 1000;
                 timestamp += offset;
                 time_api = null;
            }
            if(timestamp > 0){
                systemDate = new Date(timestamp);
            }
        }
        return systemDate;
    };
	
}