
/* Compatible models:
 *      - 
 *      - 
 *      - 
 *      - 
 *      - 
 *      - 
 *      - 
 */

function Screen() {
	
    // Reference to "this" that won't get clobbered by some other "this"
    var self = this; self.name = "LG_RS232";

    /* 
     * Timers use
     * if (timer = 1) it will load if (timer = 0) it won't load
     * Once init is called, timeouts references are stored here
     */
    this.timers = {
        power : 1,
        channel : 0,
        schedule : 1
    }

    /*
     * Port configuration:
     */
    
    self.portValues = {
    		name : '',
    	    config : {
		            baudRate : 9600,
		            dataBits : 8,
		            stopBits : 1
		    },
		    parser : {
				type : 'byteDelimiter' ,
				delimiter : [120]
		    }	
    }
    
    /******************************** INIT START STOP ********************************/
    this.init = function ( screen, platform, counter ){
    	self.data.internal_id = counter;
        self.data.id = screen.id;
        self.data.remote_id = screen.remote_id;
        self.data.formated_remote_id  = ( "00" + self.data.remote_id.toString(16)).substr(-2);
        self.data.model = screen.model;
        
        self.portValues.name = screen.port;
        self.config.schedule = self.getBinSchedule(screen.schedule);
        //console.log ("************************************************************************" )
        //console.log (platform )
        //console.log ("************************************************************************" )
        if ( platform == "WIN" ){
            self.protocol = require ('./app/screenDrivers/win/RS232.js')();
        }else if ( platform == "MAC"){
            self.protocol = require ('./app/screenDrivers/mac/RS232.js')();
        }else {
        	return false;
        }
        
        //START CONTROL TIMERS
        self.stopping = false;
    	if (self.timers.power){   
            clearTimeout ( self.timers.power );
        	self.timers.power		= setTimeout ( self.checkPowerStatus , self.intervals.power );
        }
        if (self.timers.channel){
        	clearTimeout ( self.timers.channel );
        	self.timers.channel		= setTimeout ( self.checkChannelStatus , self.intervals.channel );
        }
    	if (self.timers.schedule){
    		clearTimeout ( self.timers.schedule );
    		self.timers.schedule	= setTimeout ( self.checkSchedule , self.intervals.schedule );
    	}
        
        return true;
    }
    
    this.start = function (){
    	self.stopping = false;
    	if (self.timers.power){   
            clearTimeout ( self.timers.power );
        	self.timers.power		= setTimeout ( self.checkPowerStatus , self.intervals.power );
        }
        if (self.timers.channel){
        	clearTimeout ( self.timers.channel );
        	self.timers.channel		= setTimeout ( self.checkChannelStatus , self.intervals.channel );
        }
    	if (self.timers.schedule){
    		clearTimeout ( self.timers.schedule );
    		self.timers.schedule	= setTimeout ( self.checkSchedule , self.intervals.schedule );
    	}
    }
    
    this.stop = function (){
    	self.stopping = true;
    	if (self.timers.power){        	
            clearTimeout ( self.timers.power );
            self.timers.power = 1;
        }
        if (self.timers.channel){
        	clearTimeout ( self.timers.channel );
        	self.timers.channel = 1;
        }
    	if (self.timers.schedule){
    		clearTimeout ( self.timers.schedule );
    		self.timers.schedule = 1;
    	}        
    }
    

    
/*********************************** POWER ***********************************/
    this.checkPowerStatus = function () {
    	//clearTimeout ( self.timers.power );
    	//self.timers.power = setTimeout ( self.checkPowerStatus , self.intervals.power );
        console.log ( new Date() + ' - LG RS232, screen: '+ self.data.id + " : Start check power status // Internal_ID: " + self.data.internal_id  );
        self.protocol.sendAscii ( 'ka ' + self.data.formated_remote_id + ' ff\r' , self.portValues, self.checkPowerStatusCallback );
    }
    //Private Callback
    this.checkPowerStatusCallback = function (err, data ){
        if (err){
        	console.log ( new Date() + ' - LG RS232, screen: '+ self.data.id + " : ERROR check power status: " + err );    
        	self.errorHandling ( err );
        	
            if (self.errCounter.power < 3 ){
                self.errCounter.power++;
            }else {
                self.errCounter.power = 0;
                self.status.power = -1;
                self.status.tstamp = Math.floor(new Date().getTime()/1000);
            }
            clearTimeout ( self.timers.power );
            if (!self.stopping) {
            	self.timers.power = setTimeout ( self.checkPowerStatus , self.intervals.power );
            }
            return;
        }
        
        //Translate Result to self.status.power
        self.status.lastCMD = data.toString('ascii');
        if (data.includes('OK01') ){
            self.status.power = 1;
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes('OK00')){
            self.status.power = 0;
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else{
            self.status.power = -1;
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }
        console.log ( new Date() + ' - LG RS232, screen: '+ self.data.id + " : Result check power status: " + self.status.power );

        clearTimeout ( self.timers.power );
        if (!self.stopping) {
        	self.timers.power = setTimeout ( self.checkPowerStatus , self.intervals.power );
        }
        
    }
    
/********************************** CHANNEL **********************************/
    this.checkChannelStatus = function (){
    	//clearTimeout ( self.timers.channel );
    	//self.timers.channel = setTimeout ( self.checkChannelStatus , self.intervals.channel );
        console.log ( new Date() + ' - LG RS232, screen: '+ self.data.id + " : Start check channel status // Internal_ID: " + self.data.internal_id );
        self.protocol.sendAscii ( 'xb ' + self.data.formated_remote_id + ' ff\r' , self.portValues, self.checkChannelStatusCallback );
    }
    //Private Callback
    this.checkChannelStatusCallback = function (err, data){
        if (err){
        	console.log ( new Date() + ' - LG RS232, screen: '+ self.data.id + " : ERROR check channel status: " + err );    
        	self.errorHandling ( err );
            if (self.errCounter.channel < 3 ){
                self.errCounter.channel++;
            }else {
                self.errCounter.channel = 0;
                self.status.channel = "Err-Unknown";
                self.status.tstamp = Math.floor(new Date().getTime()/1000);
            }
        	clearTimeout ( self.timers.channel );
            if (!self.stopping) {
            	self.timers.channel = setTimeout ( self.checkChannelStatus , self.intervals.channel );
            }
            return;
        }
        console.log ( new Date() + ' - LG RS232, screen: '+ self.data.id + " : Result check channel status: " + self.status.channel );
        
        //Translate Result to self.status.channel
        if (data.includes('OK60x' )){
            self.status.channel = 'PC';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes('OK90x')){
            self.status.channel = 'HDMI1';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes('OK91x')){
            self.status.channel = 'HDMI2';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes('OK20x')){
            self.status.channel = 'AV1';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes('OK21x')){
            self.status.channel = 'AV2';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes('OK40x')){
            self.status.channel = 'COMPONENT1';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes('OK41x')){
            self.status.channel = 'COMPONENT2';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes('OK00x')){
            self.status.channel = 'DTV Antenna';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes('OK01x')){
            self.status.channel = 'DTV Cable';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes('OK10x')){
            self.status.channel = 'Analog Antenna';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes('OK11x')){
            self.status.channel = 'Analog Cable';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else{
            self.status.channel = 'Unknown';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }
        
    	clearTimeout ( self.timers.channel );
        if (!self.stopping) {
        	self.timers.channel = setTimeout ( self.checkChannelStatus , self.intervals.channel );
        }
    }
    
/********************************* SCHEDULLE *********************************/
    this.checkSchedule = function (){
    	//clearTimeout ( self.timers.schedule );
    	//self.timers.schedule = setTimeout ( self.checkSchedule , self.intervals.schedule );
        console.log ( new Date() + ' - LG RS232, screen: '+ self.data.id + " : Start check schedule // Internal_ID: " + self.data.internal_id );
        var scheduleStatus = self.getScheduledPower ( self.config.schedule , self.config.delay) ;
        /*if ( self.status.power == -1 ){
        	self.checkScheduleCallback( null, "Screen Disabled: Power -1");
        }else{*/
            if ( self.status.power != scheduleStatus ){
            	console.log ( new Date() + ' - RLY8 RS232, screen: '+ self.data.id + " : Result check schedule: Change needed");

                if ( self.getScheduledPower ( self.config.schedule , self.config.delay) ){
                    self.turnOn();
                }else{
                    self.turnOff();
                }
            }else{
            	self.checkScheduleCallback( null, "OK" );
            }
        //}
    }
    //Private Callback
    this.checkScheduleCallback = function (err, data){
        if (err){
        	console.log ( new Date() + ' - LG RS232, screen: '+ self.data.id + " : ERROR check schedule status: " + err );    
        	self.errorHandling ( err );
            clearTimeout ( self.timers.schedule );
            if (!self.stopping) {
            	self.timers.schedule = setTimeout ( self.checkSchedule , self.intervals.schedule );
            }
            return;
        }
        console.log ( new Date() + ' - LG RS232, screen: '+ self.data.id + " : Result check schedule: " + data );
        	
        clearTimeout ( self.timers.schedule );
        if (!self.stopping) {
        	self.timers.schedule = setTimeout ( self.checkSchedule , self.intervals.schedule );
        }
    }
    
    
    
/*********************************** MISC ***********************************/
    this.turnOn = function () {
        //console.log ( new Date() + ' - LG RS232, screen: '+ self.data.id + " : Start turn on" );
        self.protocol.sendAscii( 'ka ' + self.data.formated_remote_id + ' 01\r' , self.portValues, self.turnOnCallback);
    }
    //Private Callback
    this.turnOnCallback = function(err, data){
        if (err) { console.log ( new Date() + ' - LG RS232, screen: '+ self.data.id + " : ERROR turn on: " + err ); }
        else { console.log ( new Date() + ' - LG RS232, screen: '+ self.data.id + " : Result turn on: " + data ); }
        self.checkScheduleCallback( err, data);
    }
    
    this.turnOff = function () {
        //console.log ( new Date() + ' - LG RS232, screen: '+ self.data.id + " : Start turn off");
        self.protocol.sendAscii( 'ka ' + self.data.formated_remote_id + ' 00\r' , self.portValues, self.turnOnCallback);
    }
    //Private Callback
    this.turnOffCallback = function (err, data ){
        if (err) { console.log ( new Date() + ' - LG RS232, screen: '+ self.data.id + " : ERROR turn off: " + err ); }
        else { console.log ( new Date() + ' - LG RS232, screen: '+ self.data.id + " : Result turn off: " + data ); }
        self.checkScheduleCallback( err, data);
    }

    this.sendFreeCommand = function ( command ) {
        //console.log ( new Date() + ' - LG RS232, screen: '+ self.data.id + " : Start send free command");
        self.protocol.sendAscii( command , self.portValues, self.sendFreeCommandCallback);
    }
    //Private Callback
    this.sendFreeCommandCallback = function (err, data ){
        if (err) { console.log ( new Date() + ' - LG RS232, screen: '+ self.data.id + " : ERROR send free command: " + err ); }
        else { console.log ( new Date() + ' - LG RS232, screen: '+ self.data.id + " : Result send free command: " + data ); }
    }
}

Screen.prototype = new BasicScreenController();
