
/* Compatible models:
 *      - 
 *      - 
 *      - 
 *      - 
 *      - 
 *      - 
 *      - 
 */

function Screen() {
	
    // Reference to "this" that won't get clobbered by some other "this"
    var self = this; self.name = "PANASONIC_RS232";

    /* 
     * Timers use
     * if (timer = 1) it will load if (timer = 0) it won't load
     * Once init is called, timeouts references are stored here
     */
    this.timers = {
        power : 1,
        channel : 0,
        schedule : 1
    }
    
    this.retriesMaxAttempts = {
            power : 5,
            channel : 0,
            schedule : 5
    };
    
    this.retriesCounter = {
            power : 0,
            channel : 0,
            schedule : 0
    };
    
    /*
     * Port configuration:
     */
    
    self.portValues = {
    		name : '',
    	    config : {
		            baudRate : 9600,
		            dataBits : 8,
		            stopBits : 1
		    },
		    parser : {
	    		type : 'byteDelimiter' ,
	            delimiter : [03]
		    }	
    }
    var command_init = Buffer.from ( 'AD94;RAD:' , 'ascii');
    
    /******************************** INIT START STOP ********************************/
    this.init = function ( screen, platform, counter ){
    	self.data.internal_id = counter;
        self.data.id = screen.id;
        self.data.remote_id = screen.remote_id;
        self.data.formated_remote_id  = Buffer.from ( ( "000" + self.data.remote_id.toString()).substr(-3) , 'ascii');
        self.data.model = screen.model;
        
        self.portValues.name = screen.port;
        self.config.schedule = self.getBinSchedule(screen.schedule);
        //console.log ("************************************************************************" )
        //console.log (platform )
        //console.log ("************************************************************************" )
        if ( platform == "WIN" ){
            self.protocol = require ('./app/screenDrivers/win/RS232.js')();
        }else if ( platform == "MAC"){
            self.protocol = require ('./app/screenDrivers/mac/RS232.js')();
        }else {
        	return false;
        }
        
        //START CONTROL TIMERS
        self.stopping = false;
    	if (self.timers.power){   
            clearTimeout ( self.timers.power );
        	self.timers.power		= setTimeout ( self.checkPowerStatus , 5000 );
        }
        if (self.timers.channel){
        	clearTimeout ( self.timers.channel );
        	self.timers.channel		= setTimeout ( self.checkChannelStatus , 10000 );
        }
    	if (self.timers.schedule){
    		clearTimeout ( self.timers.schedule );
    		self.timers.schedule	= setTimeout ( self.checkSchedule , 2500 );
    	}
        
        return true;
    }
    
    this.start = function (){
    	self.stopping = false;
    	if (self.timers.power){   
            clearTimeout ( self.timers.power );
        	self.timers.power		= setTimeout ( self.checkPowerStatus , self.intervals.power );
        }
        if (self.timers.channel){
        	clearTimeout ( self.timers.channel );
        	self.timers.channel		= setTimeout ( self.checkChannelStatus , self.intervals.channel );
        }
    	if (self.timers.schedule){
    		clearTimeout ( self.timers.schedule );
    		self.timers.schedule	= setTimeout ( self.checkSchedule , self.intervals.schedule );
    	}
    }
    
    this.stop = function (){
    	self.stopping = true;
    	if (self.timers.power){        	
            clearTimeout ( self.timers.power );
            self.timers.power = 1;
        }
        if (self.timers.channel){
        	clearTimeout ( self.timers.channel );
        	self.timers.channel = 1;
        }
    	if (self.timers.schedule){
    		clearTimeout ( self.timers.schedule );
    		self.timers.schedule = 1;
    	}        
    }
    

    
/*********************************** POWER ***********************************/
    this.checkPowerStatus = function () {
    	self.retriesCounter.power = 0;
        console.log ( new Date() + ' - PANASONIC RS232, screen: '+ self.data.id + " : Start check power status // Internal_ID: " + self.data.internal_id  );
        var command = Buffer.from ( ';QPW' , 'ascii');
        self.protocol.sendHex ( Buffer.concat([self.STX, command_init, self.data.formated_remote_id, command, self.ETX ]) , self.portValues, self.checkPowerStatusCallback );
    }
    //Private Callback
    this.checkPowerStatusCallback = function (err, data, checkTstamp, callback ){
    	self.retriesCounter.power++;
        if (err){
        	console.log ( new Date() + ' - PANASONIC RS232, screen: '+ self.data.id + " : ERROR check power status: " + err );    
        	self.errorHandling ( err );
        	
            if ( self.retriesCounter.power < self.retriesMaxAttempts.power ){
            	callback ("retry", checkTstamp );
            }else {
            	
            	if (self.errCounter.power < 3 ){
                    self.errCounter.power++;
                }else {
                    self.errCounter.power = 0;
                    self.status.power = -1;
                    self.status.tstamp = Math.floor(new Date().getTime()/1000);
                }
            	
            	callback ("next", checkTstamp );
            	clearTimeout ( self.timers.power );
	            if (!self.stopping) {
	            	self.timers.power = setTimeout ( self.checkPowerStatus , self.intervals.power );
	            }
            }
            return;
        }
        
        self.errCounter.power = 0;
        
        //Translate Result to self.status.power
        self.status.lastCMD = data.toString('ascii');
        data = data.toString();
        if (data.includes('QPW:1') ){
            self.status.power = 1;
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
            self.retriesCounter.power = self.retriesMaxAttempts.power;
        }else if (data.includes('QPW:0')){
            self.status.power = 0;
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
            self.retriesCounter.power = self.retriesMaxAttempts.power;
        }else{
        	if ( !(self.retriesCounter.power < self.retriesMaxAttempts.power) ){
        		self.status.power = -1;
        		self.status.tstamp = Math.floor(new Date().getTime()/1000);
        	}
        }
        
        console.log ( new Date() + ' - PANASONIC RS232, screen: '+ self.data.id + " : Result check power status: " + self.status.power );    
        
        if ( self.retriesCounter.power < self.retriesMaxAttempts.power ){
        	callback ("retry", checkTstamp );
        }else {
        	callback ("next", checkTstamp );
        	clearTimeout ( self.timers.power );
            if (!self.stopping) {
            	self.timers.power = setTimeout ( self.checkPowerStatus , self.intervals.power );
            }
        }
        
    }
    
/********************************** CHANNEL **********************************/
    this.checkChannelStatus = function (){
    	self.retriesCounter.channel = 0;
    	console.log ( new Date() + ' - PANASONIC RS232, screen: '+ self.data.id + " : Start check channel status // Internal_ID: " + self.data.internal_id );
        var command = Buffer.from ( ';QMI' , 'ascii');
        self.protocol.sendHex ( Buffer.concat([self.STX, command_init, self.data.formated_remote_id, command, self.ETX ]) , self.portValues, self.checkChannelStatusCallback );
    }
    //Private Callback
    this.checkChannelStatusCallback = function (err, data, checkTstamp, callback ){
    	self.retriesCounter.channel++;
        if (err){
            console.log ( new Date() + ' - PANASONIC RS232, screen: '+ self.data.id + " : ERROR check channel status: " + err );
        	self.errorHandling ( err );
        	if ( self.retriesCounter.channel < self.retriesMaxAttempts.channel ){
            	callback ("retry", checkTstamp);
            }else {
            	if (self.errCounter.channel < 3 ){
                    self.errCounter.channel++;
                }else {
                    self.errCounter.channel = 0;
                    self.status.channel = "Err-Unknown";
                    self.status.tstamp = Math.floor(new Date().getTime()/1000);
                }
            	
            	callback ("next", checkTstamp );
	        	clearTimeout ( self.timers.channel );
	            if (!self.stopping) {
	            	self.timers.channel = setTimeout ( self.checkChannelStatus , self.intervals.channel );
	            }
            }
            return;
        }
        self.errCounter.channel = 0;
        
        console.log ( new Date() + ' - PANASONIC RS232, screen: '+ self.data.id + " : Result check channel status: " + self.status.channel );

        //Translate Result to self.status.channel
        self.status.channel = data;
        self.status.tstamp = Math.floor(new Date().getTime()/1000);
        self.retriesCounter.channel = self.retriesMaxAttempts.channel;
        
        
    	if ( self.retriesCounter.channel < self.retriesMaxAttempts.channel ){
        	callback ("retry", checkTstamp );
        }else {
        	callback ("next", checkTstamp );
	    	clearTimeout ( self.timers.channel );
	        if (!self.stopping) {
	        	self.timers.channel = setTimeout ( self.checkChannelStatus , self.intervals.channel );
	        }
        }
    }
    
/********************************* SCHEDULLE *********************************/
    this.checkSchedule = function (){
    	self.retriesCounter.schedule = 0;
        console.log ( new Date() + ' - PANASONIC RS232, screen: '+ self.data.id + " : Start check schedule // Internal_ID: " + self.data.internal_id );
        if ( self.status.power == -1 ){
        	self.checkScheduleCallback( null, "Screen Disabled: Power -1");
        }else{
            if ( self.status.power != self.getScheduledPower ( self.config.schedule , self.config.delay) ){
            	console.log ( new Date() + ' - PANASONIC RS232, screen: '+ self.data.id + " : Result check schedule: Change needed");

                if ( self.getScheduledPower ( self.config.schedule , self.config.delay) ){
                    self.turnOn();
                }else{
                    self.turnOff();
                }
            }else{
            	self.checkScheduleCallback( null, "OK");
            }
        }
    }
    //Private Callback
    this.checkScheduleCallback = function (err, data, checkTstamp, callback){
    	self.retriesCounter.schedule++;
        if (err){
        	console.log ( new Date() + ' - PANASONIC RS232, screen: '+ self.data.id + " : ERROR check schedule status: " + err );
        	self.errorHandling ( err );
        	if ( self.retriesCounter.schedule < self.retriesMaxAttempts.schedule ){
            	callback ("retry", checkTstamp );
            }else {
            	callback ("next", checkTstamp );
	        	clearTimeout ( self.timers.schedule );
	            if (!self.stopping) {
	            	self.timers.schedule = setTimeout ( self.checkSchedule , self.intervals.schedule );
	            }
            }
            return;
        }
        console.log ( new Date() + ' - PANASONIC RS232, screen: '+ self.data.id + " : Result check schedule: " + data );
        	
        if ( data == "OK" ){
        	clearTimeout ( self.timers.schedule );
	        if (!self.stopping) {
	        	self.timers.schedule = setTimeout ( self.checkSchedule , self.intervals.schedule );
	        }
	        return;
        }
        if ( self.retriesCounter.schedule < self.retriesMaxAttempts.schedule ){
        	callback ("retry", checkTstamp );
        }else {
        	callback ("next", checkTstamp );
	        clearTimeout ( self.timers.schedule );
	        if (!self.stopping) {
	        	self.timers.schedule = setTimeout ( self.checkSchedule , self.intervals.schedule );
	        }
        }
    }
    
    
    
/*********************************** MISC ***********************************/
    this.turnOn = function () {
        //console.log ( new Date() + ' - PANASONIC RS232, screen: '+ self.data.id + " : Start turn on" );
        var command = Buffer.from ( ';PON' , 'ascii');
        self.protocol.sendHex ( Buffer.concat([self.STX, command_init, self.data.formated_remote_id, command, self.ETX ]) , self.portValues, self.turnOnCallback );
    }
    //Private Callback
    this.turnOnCallback = function(err, data, checkTstamp, callback){
    	if (err) { console.log ( new Date() + ' - PANASONIC RS232, screen: '+ self.data.id + " : ERROR turn on: " + err ); }
    	else { console.log ( new Date() + ' - PANASONIC RS232, screen: '+ self.data.id + " : Result turn on: " + data ); }
    	self.checkScheduleCallback( err, data, checkTstamp, callback);
    }
    
    this.turnOff = function () {
        //console.log ( new Date() + ' - PANASONIC RS232, screen: '+ self.data.id + " : Start turn off");
        var command = Buffer.from ( ';POF' , 'ascii');
        self.protocol.sendHex ( Buffer.concat([self.STX, command_init, self.data.formated_remote_id, command, self.ETX ]) , self.portValues, self.turnOffCallback );
    }
    //Private Callback
    this.turnOffCallback = function (err, data, checkTstamp, callback ){
    	if (err) { console.log ( new Date() + ' - PANASONIC RS232, screen: '+ self.data.id + " : ERROR turn off: " + err ); }
    	else { console.log ( new Date() + ' - PANASONIC RS232, screen: '+ self.data.id + " : Result turn off: " + data ); }
    	self.checkScheduleCallback( err, data, checkTstamp, callback);
    }

    this.sendFreeCommand = function ( command ) {
        //console.log ( new Date() + ' - PANASONIC RS232, screen: '+ self.data.id + " : Start send free command");
        var command = Buffer.from ( ';' + command , 'ascii');
        self.protocol.sendHex ( Buffer.concat([self.STX, command_init, self.data.formated_remote_id, command, self.ETX ]) , self.portValues, self.sendFreeCommandCallback );
    }
    //Private Callback
    this.sendFreeCommandCallback = function (err, data, checkTstamp, callback ){
        console.log ( new Date() + ' - PANASONIC RS232, screen: '+ self.data.id + " : Result send free command: " + data );
    }



}

Screen.prototype = new BasicScreenController();
