
/* Compatible models:
 *      - 
 *      - 
 *      - 
 *      - 
 *      - 
 *      - 
 *      - 
 */

function Screen() {
	
    // Reference to "this" that won't get clobbered by some other "this"
    var self = this; self.name = "PHILIPS_RS232";

    /* 
     * Timers use
     * if (timer = 1) it will load if (timer = 0) it won't load
     * Once init is called, timeouts references are stored here
     */
    this.timers = {
        power : 1,
        channel : 0,
        schedule : 1
    }
    
    /*
     * 
     */
    
    // Empty Function, will be executed without parameters once the port is loaded
    this.customSerialParser = function () {
	    var incommingData = new Buffer(0);
	    var incommingMsgLength = 0;
		return function(emitter, buffer) {
	        incommingData = Buffer.concat([incommingData, buffer]);
	        if ( incommingMsgLength == 0 && incommingData.length > 0 ){
	        	incommingMsgLength = incommingData[0];
	        }

	        if ( incommingMsgLength != 0 && incommingData.length >= incommingMsgLength ) {
	        	var out = incommingData.slice(0,incommingMsgLength);
	        	incommingData = incommingData.slice(length);
	            emitter.emit("data", out);
	        }
	    };
	};
    
    /*
     * Port configuration:
     */
    
    this.portValues = {
    		name : '',
    	    config : {
		            baudRate : 9600,
		            dataBits : 8,
		            stopBits : 1
		    },
		    parser : {
	    		type : 'customSerialParser' ,
	            delimiter : self.customSerialParser
		    }	
    }
    
    
    //var command_init = Buffer.from ( 'AD94;RAD' , 'assci');
    var group = Buffer.from ( '00' , 'hex' );
    
    /******************************** INIT START STOP ********************************/
    this.init = function ( screen, platform, counter ){
    	self.data.internal_id = counter;
        self.data.id = screen.id;
        self.data.remote_id = screen.remote_id;
        self.data.formated_remote_id  = Buffer.from ( ( "00" + self.data.remote_id.toString()).substr(-2) , 'hex');
        self.data.model = screen.model;
        
        self.portValues.name = screen.port;
        self.config.schedule = self.getBinSchedule(screen.schedule);
        //console.log ("************************************************************************" )
        //console.log (platform )
        //console.log ("************************************************************************" )
        if ( platform == "WIN" ){
            self.protocol = require ('./app/screenDrivers/win/RS232.js')();
        }else if ( platform == "MAC"){
            self.protocol = require ('./app/screenDrivers/mac/RS232.js')();
        }else {
        	return false;
        }
        
        //START CONTROL TIMERS
        self.stopping = false;
    	if (self.timers.power){   
            clearTimeout ( self.timers.power );
        	self.timers.power		= setTimeout ( self.checkPowerStatus , self.intervals.power );
        }
        if (self.timers.channel){
        	clearTimeout ( self.timers.channel );
        	self.timers.channel		= setTimeout ( self.checkChannelStatus , self.intervals.channel );
        }
    	if (self.timers.schedule){
    		clearTimeout ( self.timers.schedule );
    		self.timers.schedule	= setTimeout ( self.checkSchedule , self.intervals.schedule );
    	}
        
        return true;
    }
    
    this.start = function (){
    	self.stopping = false;
    	if (self.timers.power){   
            clearTimeout ( self.timers.power );
        	self.timers.power		= setTimeout ( self.checkPowerStatus , self.intervals.power );
        }
        if (self.timers.channel){
        	clearTimeout ( self.timers.channel );
        	self.timers.channel		= setTimeout ( self.checkChannelStatus , self.intervals.channel );
        }
    	if (self.timers.schedule){
    		clearTimeout ( self.timers.schedule );
    		self.timers.schedule	= setTimeout ( self.checkSchedule , self.intervals.schedule );
    	}
    }
    
    this.stop = function (){
    	self.stopping = true;
    	if (self.timers.power){        	
            clearTimeout ( self.timers.power );
            self.timers.power = 1;
        }
        if (self.timers.channel){
        	clearTimeout ( self.timers.channel );
        	self.timers.channel = 1;
        }
    	if (self.timers.schedule){
    		clearTimeout ( self.timers.schedule );
    		self.timers.schedule = 1;
    	}        
    }
    

    
/*********************************** POWER ***********************************/
    this.checkPowerStatus = function () {
    	//clearTimeout ( self.timers.power );
    	//self.timers.power = setTimeout ( self.checkPowerStatus , self.intervals.power );
        //console.log ( new Date() + ' - PHILIPS RS232, screen: '+ self.data.id + " : Start check power status // Internal_ID: " + self.data.internal_id  );
        var command = Buffer.from ( '19' , 'hex');
        var length = Buffer.alloc(1);
        length[0] = 4 + command.length;
        command = Buffer.concat([ length, self.data.formated_remote_id, group, command ]);
        command = Buffer.concat([ command, self.getXorChecksum(command) ]);
        self.protocol.sendHex ( command , self.portValues, self.checkPowerStatusCallback );
    }
    //Private Callback
    this.checkPowerStatusCallback = function (err, data ){
        if (err){
        	console.log ( new Date() + ' - PHILIPS RS232, screen: '+ self.data.id + " : ERROR check status status: " + err );
        	self.errorHandling ( err );
            if (self.errCounter.power < 3 ){
                self.errCounter.power++;
            }else {
                self.errCounter.power = 0;
                self.status.power = -1;
                self.status.tstamp = Math.floor(new Date().getTime()/1000);
            }
            clearTimeout ( self.timers.power );
            if (!self.stopping) {
            	self.timers.power = setTimeout ( self.checkPowerStatus , self.intervals.power );
            }
            return;
        }
        
        //Translate Result to self.status.power
        self.status.lastCMD = data.toString('ascii');
        data = data.toString();
        if (data.includes( Buffer.from ( '1902', 'hex') ) ){
            self.status.power = 1;
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from ( '1901', 'hex') )){
            self.status.power = 0;
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else{
            self.status.power = -1;
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }
        
        console.log ( new Date() + ' - PHILIPS RS232, screen: '+ self.data.id + " : Result check power status: " + self.status.power );    
        
        clearTimeout ( self.timers.power );
        if (!self.stopping) {
        	self.timers.power = setTimeout ( self.checkPowerStatus , self.intervals.power );
        }
        
    }
    
/********************************** CHANNEL **********************************/
    this.checkChannelStatus = function (){
    	//clearTimeout ( self.timers.channel );
    	//self.timers.channel = setTimeout ( self.checkChannelStatus , self.intervals.channel );
        //console.log ( new Date() + ' - PHILIPS RS232, screen: '+ self.data.id + " : Start check channel status // Internal_ID: " + self.data.internal_id );
        var command = Buffer.from ( 'AD' , 'hex');
        var length = Buffer.alloc(1);
        length[0] = 4 + command.length;
        command = Buffer.concat([ length, self.data.formated_remote_id, group, command ]);
        command = Buffer.concat([ command, self.getXorChecksum(command) ]);
        self.protocol.sendHex ( command , self.portValues, self.checkPowerStatusCallback );
    }
    //Private Callback
    this.checkChannelStatusCallback = function (err, data){
        if (err){
        	console.log ( new Date() + ' - PHILIPS RS232, screen: '+ self.data.id + " : ERROR check channel status: " + err );
        	self.errorHandling ( err );
            if (self.errCounter.channel < 3 ){
                self.errCounter.channel++;
            }else {
                self.errCounter.channel = 0;
                self.status.channel = "Err-Unknown";
                self.status.tstamp = Math.floor(new Date().getTime()/1000);
            }
        	clearTimeout ( self.timers.channel );
            if (!self.stopping) {
            	self.timers.channel = setTimeout ( self.checkChannelStatus , self.intervals.channel );
            }
            return;
        }
        console.log ( new Date() + ' - PHILIPS RS232, screen: '+ self.data.id + " : Result check channel status: " + self.status.channel );

        //Translate Result to self.status.channel
        if (data.includes( Buffer.from( 'AD01', 'hex') )){
            self.status.channel = 'VIDEO';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD02', 'hex') )){
            self.status.channel = 'S-VIDEO';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD03', 'hex') )){
            self.status.channel = 'COMPONENT';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD04', 'hex') )){
            self.status.channel = 'CVI 2';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD05', 'hex') )){
            self.status.channel = 'VGA';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD06', 'hex') )){
            self.status.channel = 'HDMI 2';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD07', 'hex') )){
            self.status.channel = 'Display Port 2';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD08', 'hex') )){
            self.status.channel = 'USB 2';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD09', 'hex') )){
            self.status.channel = 'Card DVI-D';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD0A', 'hex') )){
            self.status.channel = 'Display Port 1';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD0B', 'hex') )){
            self.status.channel = 'Card OPS';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD0C', 'hex') )){
            self.status.channel = 'USB 1';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD0D', 'hex') )){
            self.status.channel = 'HDMI 1';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD0E', 'hex') )){
            self.status.channel = 'DVI-D';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD0F', 'hex') )){
            self.status.channel = 'HDMI 3';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD10', 'hex') )){
            self.status.channel = 'BROWSER';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD11', 'hex') )){
            self.status.channel = 'SMARTCMS';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD12', 'hex') )){
            self.status.channel = 'DMS (Digital Media Server)';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD13', 'hex') )){
            self.status.channel = 'INTERNAL STORAGE';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD14', 'hex') )){
            self.status.channel = 'Reserved';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD15', 'hex') )){
            self.status.channel = 'Reserved';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD16', 'hex') )){
            self.status.channel = 'Media Player';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD17', 'hex') )){
            self.status.channel = 'PDF Player';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else if (data.includes( Buffer.from( 'AD18', 'hex') )){
            self.status.channel = 'Custom';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }else{
            self.status.channel = 'Unknown';
            self.status.tstamp = Math.floor(new Date().getTime()/1000);
        }
        
    	clearTimeout ( self.timers.channel );
        if (!self.stopping) {
        	self.timers.channel = setTimeout ( self.checkChannelStatus , self.intervals.channel );
        }
    }
    
/********************************* SCHEDULLE *********************************/
    this.checkSchedule = function (){
    	//clearTimeout ( self.timers.schedule );
    	//self.timers.schedule = setTimeout ( self.checkSchedule , self.intervals.schedule );
        //console.log ( new Date() + ' - PHILIPS RS232, screen: '+ self.data.id + " : Start check schedule // Internal_ID: " + self.data.internal_id );
        if ( self.status.power == -1 ){
        	self.checkScheduleCallback( null, "Screen Disabled: Power -1");
        }else{
            if ( self.status.power != self.getScheduledPower ( self.config.schedule , self.config.delay) ){
            	console.log ( new Date() + ' - PHILIPS RS232, screen: '+ self.data.id + " : Result check schedule: Change needed");

                if ( self.getScheduledPower ( self.config.schedule , self.config.delay) ){
                    self.turnOn();
                }else{
                    self.turnOff();
                }
            }else{
            	self.checkScheduleCallback( null, "OK");
            }
        }
    }
    //Private Callback
    this.checkScheduleCallback = function (err, data){
        if (err){
        	console.log ( new Date() + ' - PHILIPS RS232, screen: '+ self.data.id + " : ERROR check schedule status: " + err );
        	self.errorHandling ( err );
            clearTimeout ( self.timers.schedule );
            if (!self.stopping) {
            	self.timers.schedule = setTimeout ( self.checkSchedule , self.intervals.schedule );
            }
            return;
        }
        console.log ( new Date() + ' - PHILIPS RS232, screen: '+ self.data.id + " : Result check schedule: " + data );
        	
        clearTimeout ( self.timers.schedule );
        if (!self.stopping) {
        	self.timers.schedule = setTimeout ( self.checkSchedule , self.intervals.schedule );
        }
    }
    
    
    
/*********************************** MISC ***********************************/
    this.turnOn = function () {
        //self.screenData.status = 1;
        //console.log ( new Date() + ' - PHILIPS RS232, screen: '+ self.data.id + " : Start turn on" );
        var command = Buffer.from ( '1802' , 'hex');
        var length = Buffer.alloc(1);
        length[0] = 4 + command.length;
        command = Buffer.concat([ length, self.data.formated_remote_id, group, command ]);
        command = Buffer.concat([ command, self.getXorChecksum(command) ]);
        self.protocol.sendHex ( command , self.portValues, self.checkPowerStatusCallback );
    }
    //Private Callback
    this.turnOnCallback = function(err, data){
        console.log ( new Date() + ' - PHILIPS RS232, screen: '+ self.data.id + " : Result turn on: " + data );
        self.checkScheduleCallback( err, data);
    }
    
    this.turnOff = function () {
        //console.log ( new Date() + ' - PHILIPS RS232, screen: '+ self.data.id + " : Start turn off");
        var command = Buffer.from ( '1801' , 'hex');
        var length = Buffer.alloc(1);
        length[0] = 4 + command.length;
        command = Buffer.concat([ length, self.data.formated_remote_id, group, command ]);
        command = Buffer.concat([ command, self.getXorChecksum(command) ]);
        self.protocol.sendHex ( command , self.portValues, self.checkPowerStatusCallback );
    }
    //Private Callback
    this.turnOffCallback = function (err, data ){
        console.log ( new Date() + ' - PHILIPS RS232, screen: '+ self.data.id + " : Result turn off: " + data );
        self.checkScheduleCallback( err, data);
    }

    this.sendFreeCommand = function ( command ) {
        //console.log ( new Date() + ' - PHILIPS RS232, screen: '+ self.data.id + " : Start send free command");
        var command = Buffer.from ( command , 'hex');
        var length = Buffer.alloc(1);
        length[0] = 4 + command.length;
        command = Buffer.concat([ length, self.data.formated_remote_id, group, command ]);
        command = Buffer.concat([ command, self.getXorChecksum(command) ]);
        self.protocol.sendHex ( command , self.portValues, self.checkPowerStatusCallback );
    }
    //Private Callback
    this.sendFreeCommandCallback = function (err, data ){
        console.log ( new Date() + ' - PHILIPS RS232, screen: '+ self.data.id + " : Result send free command: " + data );
    }



}

Screen.prototype = new BasicScreenController();
