module.exports = function() {
  return new protocol();
}


// The actual constructor function, which is also a closure 
// holding all the private state information
function protocol() {
	if(arguments.callee._singletonInstance){
	    return arguments.callee._singletonInstance;
	}
	arguments.callee._singletonInstance = this;

	var self = this; self.name = "serialport-mac"; // MANDATORY
	self.serialPort = require ('serialport-mac');
	
	this.port = false;
	
	self.queue = [];
    self.busy = false;
    self.step = 0;
    self.currentAction = null;
    
    self.checkTstamp = new Date().getTime();

    self.shutdownPortTimer = null;
    self.shutDownPortTimeout = 2000;
    
    self.validationTimer = null;
    self.validationTimeout = 2000;
    
    
    self.writeTimer = null;
    self.writeTimeout1 = 300;
    self.writeTimeout2 = 500;
    
	self.processQueueTimer = null;
	self.processQueueTimeout = 500;
	
	self.processStepTimer = null;
	self.processStepTimeout = 500
	
	self.processCommandTimer = null;
	self.processCommandTimeout = 500;
	
	
    self.resetCounter = 0;
    self.resetCounterMax = 5;
    self.resetTimeout = 5000;
    
    
    /*********************************** ACCES FUNCTIONS ***********************************/
    this.sendAscii = function ( command, port, callback) {
        //TODO proces msg
        self.queue.push([[command, port, callback]]);
        if (self.busy) return;
        self.busy = true;
        processQueue();
    }
    this.sendHex = function ( command, port, callback ) {
        //TODO proces msg
        self.queue.push([[command, port, callback]]);
        if (self.busy) return;
        self.busy = true;
        processQueue();
    }

    this.sendAsciiDouble = function ( command, port, callback) {
        //TODO proces msg
        self.queue.push([[command, port, function(err, data, checkTstamp, callback){}]]);
        self.queue.push([[command, port, callback]]);
        if (self.busy) return;
        self.busy = true;
        processQueue();
    }
    
    this.sendAsciiSteps = function ( commandArray, portArray, callbackArray){
    	//TODO
    	var finalCommand = [];
    	for ( var i = 0; i < commandArray.length; i++ ){
    		finalCommand [i] = [ commandArray[i], portArray[i], callbackArray[i] ]
    	}
        self.queue.push( finalCommand );
        if (self.busy) return;
        self.busy = true;
        processQueue();
    }

    // Private functions can exist in the closure too; they can see the state but
    // outsiders can't touch them

    //function private() { ... }
    /*********************************** QUEUE FUNCTION ***********************************/
    function processQueue (){
    	console.log("Validation: " + "processQueue")
    	self.step = 0;
        var next = self.queue.shift();
        if (!next) {
            self.busy = false;
            return;
        }
        console.log ( "Pending Comands : " + self.queue.length)
        self.currentAction = next;
        
        processCommand();
    }
    
    function processStep () {
    	self.step ++;
    	processCommand ();
    }
    
    function processCommand () {
    	console.log("Validation: " + "processCommand")
    	self.checkTstamp = new Date().getTime();
    	
    	
    	if ( self.resetCounter > self.resetCounterMax){
    		console.log("Validation: " + "Driver Down" );
    		self.resetCounter = 0;
    		self.serialPort = null;
    		purgeCache ('serialport-mac');
    		self.serialPort = require ('serialport-mac');
    		clearTimeouts ();
            self.processCommandTimer = setTimeout (processCommand, self.processCommandTimeout);
            console.log("Validation: " + " Driver Up" );
            return;
    	}
    	self.resetCounter++;
    	
    	
    	if ( self.port ) console.log ( "Port status: " + self.port.isOpen() )
    	if ( self.port && self.port.isOpen() ){
    		console.log ( "Port still open" )
    		self.port.close();
    		clearTimeouts ();
            self.processQueueTimer = setTimeout (processCommand, self.processCommandTimeout);
    		return;
    	}
    	
    	clearTimeouts ();
    	self.shutdownPortTimer = setTimeout (shutDownPort, self.shutDownPortTimeout);
        try {
        	self.currentAction[self.step][1].config.parser = loadParser (self.currentAction[self.step][1].parser.type, self.currentAction[self.step][1].parser.delimiter )
        	self.port = new self.serialPort(self.currentAction[self.step][1].name, self.currentAction[self.step][1].config );
        }catch ( err ){
        	try {
            	clearTimeouts ();
    			self.validationTimer = setTimeout (validationFail, self.validationTimeout);
            	self.currentAction[self.step][2](err, null, self.checkTstamp, validation); //calback (error, data)
            }catch ( error ){
            }
        	return;
        }
        
        loadCallbacks ();
        self.port.on('open', function (err){
            if (err){
            	try {
            		clearTimeouts ();
                	self.validationTimer = setTimeout (validationFail, self.validationTimeout);
                	self.currentAction[self.step][2](err, null, self.checkTstamp, validation); //calback (error, data)
                }catch ( error ){
                }
                return;
            }
            clearTimeout ( self.writeTimer );
            self.writeTimer = setTimeout ( writeOnPort (), self.writeTimeout1 );
        });
        
    };
    
    
    /*********************************** WRITING FUNCTIONS ***********************************/

    function writeOnPort() {
    	clearTimeout ( self.shutdownPortTimer );
    	self.shutdownPortTimer = setTimeout (shutDownPort, self.shutDownPortTimeout);
        console.log ( new Date() + ' - Write on RS232, port: '+ self.currentAction[self.step][1].name + " : " );
        console.log ( self.currentAction[self.step][0] );
        console.log ("End command ");
        self.port.write(self.currentAction[self.step][0], function (err){
        	if (err){
        		if ( err.message.includes ("Port is not open") ){
	        		retryWriteOnPort();
	        	}
	        	else {
		    		try {
		    			clearTimeouts ();
		    			self.validationTimer = setTimeout (validationFail, self.validationTimeout);
		            	self.currentAction[self.step][2](err, null, self.checkTstamp, validation); //calback (error, data)
		            }catch ( error ){
		            }
		            if ( self.port.isOpen() ){
		                self.port.close();
		            }
	            }
    		}
        });
    }
    
    this.retry = 0;
    function retryWriteOnPort () {
    	if ( self.retry < 3 ){
    		self.retry++;
    		clearTimeout ( self.writeTimer );
            self.writeTimer = setTimeout ( writeOnPort, self.writeTimeout2 );
    	}else {
    		try {
    			clearTimeouts ();
    			self.validationTimer = setTimeout (validationFail, self.validationTimeout);
            	self.currentAction[self.step][2](new Error("Out of retries"), null, self.checkTstamp, validation); //calback (error, data)
            }catch ( error ){
            }
            if ( self.port.isOpen() ){
                self.port.close();
            }
    	}
    }
    
    /*********************************** PARSER FUNCTION ***********************************/

    function loadParser ( type, delimiter ) {
        switch (type){
            case 'readline':
                return self.serialPort.parsers.readline(delimiter)
                break;
            case 'byteDelimiter':
            	return self.serialPort.parsers.byteDelimiter(delimiter)
                break;
            case 'byteLength':
            	return self.serialPort.parsers.byteLength(delimiter)
                break;
            case 'customSerialParser':
            	return delimiter();
                break;
            default :
            	return self.serialPort.parsers.raw
                break;
        }
    }
    
    
    /*********************************** DRIVER FUNCTIONS ***********************************/
    function loadCallbacks (){
        self.port.on('close', function (){
        	
        });
        
        self.port.on('error', function(err) {
            //console.log ( new Date() + ' - Error on RS232, port: '+ err.message + " : \n\t\t\t\t\t\t" + err + "\n\t\t\t\t\tEnd error ");
            try {
            	clearTimeouts ();
    			self.validationTimer = setTimeout (validationFail, self.validationTimeout);
            	self.currentAction[self.step][2](err, null, self.checkTstamp, validation); //calback (error, data)
            }catch ( error ){
            }
            if ( self.port.isOpen() ){
                self.port.close();
            }
        });
        self.port.on('data', function (data) {
        	data = Buffer.from(data);
            //console.log ( new Date() + ' - Data on RS232, port: '+ self.currentAction[self.step][1].name + " : \n\t\t\t\t\t\t" + data + "\n\t\t\t\t\tEnd data ");
            if (!self.currentAction) return;
            try {
            	clearTimeouts ();
    			self.validationTimer = setTimeout (validationFail, self.validationTimeout);
            	self.currentAction[self.step][2](null, data, self.checkTstamp, validation); //calback (error, data)
            }catch ( error ){
            }
            if ( self.port.isOpen() ){
                self.port.close();
            }
        });
    }
    
    function shutDownPort (){
    	clearTimeout ( self.shutdownPortTimer );
    	try {
    		clearTimeouts ();
			self.validationTimer = setTimeout (validationFail, self.validationTimeout);
        	self.currentAction[self.step][2](new Error("Forced Time out"), null, self.checkTstamp, validation); //calback (error, data)
    	}catch ( error ){
        }
        if ( self.port.isOpen() ){
            //console.log ( new Date() + ' - Timeout on RS232, port: '+ self.currentAction[self.step][1].name + " : \n\t\t\t\t\t\t" + "NO DATA RECIVED" + "\n\t\t\t\t\tEnd Timeout ");
            self.port.close();
        }
    }
    
    /*********************************** RESPONSE FUNCTIONS ***********************************/
    
    function validation ( action, sendCheckTstamp, callback ){
    	console.log("Validation: " + sendCheckTstamp + ", " + action);
    	console.log ("Validation: " + self.currentAction[self.step][0] )
    	if (sendCheckTstamp == self.checkTstamp ){
    		if ( action == "next"){
                if ( self.port.isOpen() ){
            		self.port.close();
                }
                console.log("Validation: " + "executeNext")
    	    	self.checkTstamp = new Date().getTime();
    	    	self.writeRetry = 0;
    	    	clearTimeouts();
    			self.processQueueTimer = setTimeout (processQueue, self.processQueueTimeout);
    		}else if (action == "step"){
                if ( self.port.isOpen() ){
            		self.port.close();
                }
    	    	self.checkTstamp = new Date().getTime();
    	    	self.writeRetry = 0;
    			clearTimeouts();
    			self.processStepTimer = setTimeout (processStep, self.processStepTimeout);
    		}else if (action == "retry"){
                if ( self.port.isOpen() ){
            		self.port.close();
                }
    	    	self.checkTstamp = new Date().getTime();
    	    	self.writeRetry = 0;
    			clearTimeouts();
    			self.processCommandTimer = setTimeout (processCommand, self.processCommandTimeout);
    		}else{
    			console.log ( "Validation: error in action, consider fail.")
    			clearTimeouts ();
    			validationFail();
    		}
    	}
		else {
    		if (action != "next"){
    			callback ();
    		}
    	}
	}
    
	function validationFail (){
		console.log("Validation Fail: " + self.checkTstamp);
    	console.log ("Validation: " + self.currentAction[self.step][0] )
    	self.checkTstamp = new Date().getTime();
    	self.writeRetry = 0;
    	clearTimeouts();
    	self.processQueueTimer = setTimeout (processQueue, self.processQueueTimeout);
    }
    
	function clearTimeouts () {
		console.log("Validation: " + "timeouts start")
    	clearTimeout ( self.shutdownPortTimer );
    	clearTimeout ( self.validationTimer );
    	clearTimeout ( self.writeTimer );
    	clearTimeout ( self.processQueueTimer );
    	clearTimeout ( self.processStepTimer );
    	clearTimeout ( self.processCommandTimer );
    	console.log("Validation: " + "timeouts end")
    	return;
    }
    
    /*********************************** CACHE FUNCTIONS ***********************************/
    /**
     * Removes a module from the cache
     */
    function purgeCache(moduleName) {
        // Traverse the cache looking for the files
        // loaded by the specified module name
        searchCache(moduleName, function (mod) {
            delete require.cache[mod.id];
        });

        // Remove cached paths to the module.
        // Thanks to @bentael for pointing this out.
        Object.keys(module.constructor._pathCache).forEach(function(cacheKey) {
            if (cacheKey.indexOf(moduleName)>0) {
                delete module.constructor._pathCache[cacheKey];
            }
        });
    };

    /**
     * Traverses the cache to search for all the cached
     * files of the specified module name
     */
    function searchCache(moduleName, callback) {
        // Resolve the module identified by the specified name
        var mod = require.resolve(moduleName);

        // Check if the module has been resolved and found within
        // the cache
        if (mod && ((mod = require.cache[mod]) !== undefined)) {
            // Recursively go over the results
            (function traverse(mod) {
                // Go over each of the module's children and
                // traverse them
                mod.children.forEach(function (child) {
                    traverse(child);
                });

                // Call the specified callback providing the
                // found cached module
                callback(mod);
            }(mod));
        }
    };
}