/* 
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

var SerialPort = require ('serialport-mac');

module.exports = function() {
  return new protocol();
}


// The actual constructor function, which is also a closure 
// holding all the private state information
function protocol() {
	
	if(arguments.callee._singletonInstance){
	    return arguments.callee._singletonInstance;
	}
	arguments.callee._singletonInstance = this;
	
	
    // Reference to "this" that won't get clobbered by some other "this"
    var self = this; self.name = "RS232-MAC"; // MANDATORY
    
    // Private state variables
    self.portConf = {
        baudRate : 9600,
        dataBits : 8,
        stopBits : 1,
        parser: SerialPort.parsers.byteDelimiter([03])
        //parity and flow are none by default
    }

    self.portConfig = {};
    self.portName = '';
    self.port = null;
    self.queue = [];
    self.busy = false;
    self.currentAction = null;
    
    self.parserType = '';
    self.parserDelimiter = null;
    self.resetParserFunction = function (){ return function(emitter, buffer) {} };
    self.shutdownPortTimer = null;
    self.shutDownPortTimeout = 3000;
    // Public method: initialize the object
    //self.init = function(options, callback) { ... }
    self.init = function ( portName, config, parserType, parserDelimiter ){
        self.portName = portName;
        self.portConfig = config;
        self.parserType = parserType;
        self.parserDelimiter = parserDelimiter;
        loadParser();
    }
    
    self.sendAscii = function ( command , callback ) {
        //TODO proces msg
        self.queue.push([command, callback]);
        if (self.busy) return;
        self.busy = true;
        processQueue();
    }
    self.sendHex = function ( command , callback ) {
        //TODO proces msg
        self.queue.push([command, callback]);
        if (self.busy) return;
        self.busy = true;
        processQueue();
    }


    // Private functions can exist in the closure too; they can see the state but
    // outsiders can't touch them

    //function private() { ... }
    function processQueue (){
        var next = self.queue.shift();
        if (!next) {
            self.busy = false;
            return;
        }

        self.currentAction = next;
        
        self.port = new SerialPort(self.portName, self.portConfig );
        loadCallbacks ();
        self.port.on('open', function (err){
            if (err){
                console.log(err);
                return;
            }
            self.shutdownPortTimer = setTimeout (function () { shutDownPort(); }, self.shutDownPortTimeout);
            console.log ( new Date() + ' - Write on RS232, port: '+ self.portName + " : \n\t\t\t\t\t\t" + self.currentAction[0] + "\n\t\t\t\t\tEnd command ");
            self.port.write(self.currentAction[0]);
        });
        
    };
    
    function loadCallbacks (){
        self.port.on('close', function (){
            self.port = null;
            clearTimeout ( self.shutdownPortTimer );
            processQueue();
        });
        self.port.on('error', function(err) {
            console.log ( new Date() + ' - Error on RS232, port: '+ err.message + " : \n\t\t\t\t\t\t" + err + "\n\t\t\t\t\tEnd error ");
            self.currentAction[1](err, null); //calback (error, data)
            if ( self.port.isOpen() ){
                self.port.flush( function (err) {
                    if (err){
                        console.log(err);
                        return;
                    }
                    resetParser();
                    self.port.close(); 
                });
            }else{
                processQueue();
            }
        });
        self.port.on('data', function (data) {
            console.log ( new Date() + ' - Data on RS232, port: '+ self.portName + " : \n\t\t\t\t\t\t" + data + "\n\t\t\t\t\tEnd data ");
            if (!self.currentAction) return;
            self.currentAction[1](null, data); //calback (error, data)
            self.port.flush( function (err) {
                if (err){
                    console.log(err);
                    return;
                }
                resetParser();
                self.port.close(); 
            });
        });
    }
    
    function loadParser () {
        switch (self.parserType){
            case 'readline':
                self.portConfig.parser = SerialPort.parsers.readline(self.parserDelimiter)
                break;
            case 'byteDelimiter':
                self.portConfig.parser =  SerialPort.parsers.byteDelimiter(self.parserDelimiter)
                break;
            case 'byteLength':
                self.portConfig.parser = SerialPort.parsers.byteLength(self.parserDelimiter)
                break;
            case 'customSerialParser':
                self.portConfig.parser = self.parserDelimiter;
                break;
            default :
                self.portConfig.parser = SerialPort.parsers.raw
                break;
        }
        /*
        if (parserType == 'raw'){
            self.portConfig.parser = SerialPort.parsers.raw
        }else if (parserType == 'readline'){
            self.portConfig.parser = SerialPort.parsers.readline(parserDelimiter)
        }else if (parserType == 'byteDelimiter'){
            self.portConfig.parser =  SerialPort.parsers.byteDelimiter(parserDelimiter)
        }else if (parserType == 'byteLength'){
            self.portConfig.parser = SerialPort.parsers.byteLength(parserDelimiter)
        }else if (parserType == 'customSerialParser'){
            self.portConfig.parser = parserDelimiter;
        }*/
    }
    
    function resetParser () {
        self.portConfig.parser = self.resetParserFunction;
        loadParser();
    }
    function shutDownPort (){
    	self.currentAction[1]("Forced Time out", null); //calback (error, data)
        if ( self.port.isOpen() ){
            self.port.flush( function (err) {
                if (err){
                    console.log(err);
                    return;
                }
                resetParser();
                console.log ( new Date() + ' - Timeout on RS232, port: '+ self.portName + " : \n\t\t\t\t\t\t" + "NO DATA RECIVED" + "\n\t\t\t\t\tEnd Timeout ");
                self.port.close(); 
            });
        }
    }
}